﻿using UnityEngine;
using System.Collections;

public class Director : Singletone<Director> {
	
	public GameObject[] portalPrefabs;
	public int hp = 30;
	public GameObject hpTxt;
	public GameObject waveTxt;
	public GameObject scorePanel;
	public GameObject[] scoreTxt;
	private int currentWave = 1;
	private int score;
	private int summaryKills = 0;
	private int waveKills;
	private float waveTimeBonus;
	private bool pause = false;
	private bool gameOver = false;

	// Use this for initialization
	void Start () {
		RefreshHpTxt();
		CreateNewWave();
		RefreshWaveTxt();
	}
	
	// Update is called once per frame
	void Update () {
		
		int enemiesLeft = GameObject.FindGameObjectsWithTag("Enemy").Length;
		int portalsLeft = GameObject.FindGameObjectsWithTag("Portal").Length;
		//Debug.Log(enemiesLeft + " " + portalsLeft);

		if (!gameOver && hp <= 0)
		{
			gameOver = true;
			Time.timeScale = 0;
			StartCoroutine("ShowScore");
		}

		if (!pause && enemiesLeft == 0 && portalsLeft == 0)
		{
			StartCoroutine("ShowScore");
		}

		if (!pause)
		{
			waveTimeBonus -= Time.deltaTime;
		}

		if (Input.GetKeyDown(KeyCode.R))
		{
			StopAllCoroutines();
			Time.timeScale = 1;
			Application.LoadLevel(Application.loadedLevel);
		}
	}

	public void LowerHp(int value)
	{
		hp -= value;
		RefreshHpTxt();
	}

	public void AddKill(int value)
	{
		waveKills++;
		score += 5;
	}

	void RefreshHpTxt()
	{
		hpTxt.GetComponent<GUIText>().text = hp.ToString();
	}

	void RefreshWaveTxt()
	{
		waveTxt.GetComponent<GUIText>().text = "Wave: " + currentWave.ToString();
	}

	void RefreshScore()
	{
		scoreTxt[0].GetComponent<GUIText>().text = "Score: " + score.ToString();
		scoreTxt[1].GetComponent<GUIText>().text = "Kills: " + summaryKills.ToString() + " (+" + waveKills.ToString() + ")";
		scoreTxt[2].GetComponent<GUIText>().text = "Time bonus: " + waveTimeBonus.ToString();
	}

	IEnumerator ShowScore() {
		pause = true;
		if (gameOver)
		{
			waveTimeBonus = 0;
			scoreTxt[3].SetActive(true);
		}
		else
		{
			waveTimeBonus = (int) waveTimeBonus;
			if (waveTimeBonus<0) waveTimeBonus=0;
			score += (int)waveTimeBonus;
		}
		summaryKills += waveKills;
		scorePanel.SetActive(true);
		AudioManager.Instance.PlaySound(AudioManager.Sound.Complete);
		currentWave++;
		RefreshWaveTxt();
		RefreshScore();
		if (!gameOver)
		{
			yield return new WaitForSeconds(3f);
			scorePanel.SetActive(false);
			CreateNewWave();
			pause = false;
		}
	}

	void CreateNewWave()
	{
		waveTimeBonus = currentWave*10+5;
		waveKills = 0;
		int portalsCount;
		if (currentWave < 3) portalsCount = 1;
		else if (currentWave < 7) portalsCount = 2;
		else if (currentWave < 13) portalsCount = 3;
		else if (currentWave < 19) portalsCount = 4;
		else portalsCount = 5;
		int difficulty = currentWave*5 + currentWave*currentWave/2 - 1;
		CreatePortals(portalsCount, difficulty/portalsCount);
	}

	public void CreatePortals(int number, int difficulty)
	{
		for (int worldLoop = 0; worldLoop < 2; worldLoop++)
		{
			for (int countLoop = 0; countLoop < number; countLoop++)
			{
				Vector3 randomPosition;
				Collider2D obstacle;
				do
				{
					randomPosition = new Vector3(Random.Range(-15f, 15.5f), Random.Range(-7.5f, 7.5f), 0);
					obstacle = Physics2D.OverlapCircle(randomPosition,1.35f,1<<(10+worldLoop));
				} while (obstacle);
				GameObject createdPortal = GameObject.Instantiate(portalPrefabs[worldLoop], randomPosition, new Quaternion()) as GameObject;
				createdPortal.GetComponent<Portal>().SetDifficulty(difficulty);
			}
		}
	}
}
